
use strict;
package MT::Plugin::TinyTuring;
use base qw(MT::Plugin);

use vars qw( $VERSION );
$VERSION = '1.02';

my $plugin_key = 'TinyTuring';
my @words = ('avoid', 'beat', 'cancel', 'defeat', 'eliminate', 'forbid', 'get rid of', 'hinder', 'inhibit', 'jettison', 'kick out', 'liquidate', 'mitigate', 'negate', 'obliterate', 'prevent', 'quash', 'reduce', 'smash', 'terminate', 'unload', 'veto', 'wipe out', 'x out', 'you can help stop', 'zap');

require MT::Plugin;
require MT;

my $tinyturing = MT::Plugin::TinyTuring->new({
	name => "TinyTuring",
	description => 'Simple human-verification test to fight comment spam.',
	author_name => 'Kevin Shay',
	author_link => 'http://www.staggernation.com/',
	version => $VERSION,
	config_template => \&_config_template,
	settings => new MT::PluginSettings([
		['enabled', { 'Default' => 1 }],
		['rejected', { 'Default' => 'junk' }],
		['salt', { 'Default' => 'xx' }]
	])
});
MT->add_plugin($tinyturing);

use MT::Template::Context;
MT::Template::Context->add_tag('TinyTuring' => sub{&tinyturing;});
MT::Template::Context->add_tag('TinyTuringStart' => sub{&tt_start;});
MT::Template::Context->add_tag('TinyTuringLetter' => sub{&tt_letter;});
MT::Template::Context->add_tag('TinyTuringWord' => sub{&tt_word;});
MT::Template::Context->add_tag('TinyTuringField' => sub{&tt_field;});
MT::Template::Context->add_tag('TinyTuringHiddenField' => sub{&tt_hidden_field;});
use MT::JunkFilter qw( ABSTAIN );
MT->register_junk_filter([
	{ code => \&tt_junkfilter, plugin => $tinyturing, name => 'TinyTuring' }
]);
require MT::App::Comments;
	# store pointer to the original subroutine
my $mt_make_comment = \&MT::App::Comments::_make_comment;
my $mt_post = \&MT::App::Comments::post;
	# install our new subroutine
{
	local $SIG{__WARN__} = sub {  }; 
	*MT::App::Comments::_make_comment = \&make_comment;
	*MT::App::Comments::post = \&post;
}

	# cache settings to avoid repeated loading
my $blog_settings = {};

sub _config_template {
	return <<HTML;
<div class="setting">
<div class="label">
<label for="enabled"><MT_TRANS phrase="Enable:"></label>
</div>
<div class="field">
<p><input value="1" type="checkbox" name="enabled" id="enabled" <TMPL_IF NAME=ENABLED>checked="checked"</TMPL_IF> />
<MT_TRANS phrase="Require TinyTuring verification for comment posting"></p>
</div> 
</div>
<div class="setting">
<div class="label">
<label for="rejected"><MT_TRANS phrase="Unverified Comments:"></label>
</div>
<div class="field">
<ul>
<li><input type="radio" name="rejected" value="junk"<TMPL_IF REJECTED_JUNK> checked="checked"</TMPL_IF> /> <MT_TRANS phrase="Junk (poster will see &quot;held for approval&quot; message)"></li>
<li><input type="radio" name="rejected" value="reject" <TMPL_IF REJECTED_REJECT> checked="checked"</TMPL_IF> /> <MT_TRANS phrase="Reject (poster will see &quot;please enter correct letter&quot; error)"></li>
</ul>
</div>
</div>
<div class="setting">
<div class="label">
<label for="salt"><MT_TRANS phrase="Salt:"></label>
</div>
<div class="field">
<p><input type="text" name="salt" value="<TMPL_VAR NAME=SALT>" maxlength="2" style="width:20px;" />
<MT_TRANS phrase="Two-letter code used to encrypt hidden field as an added safeguard"></p>
</div> 
</div>
HTML
}

sub make_comment {
    my ($app) = @_;
    my ($comment, $commenter) = &{$mt_make_comment}(@_);
    my $good = 0;
	$comment->{'TinyTuring_ok'} = tt_test($app);
	return ($comment, $commenter);
}

sub post {
	my ($app) = @_;
	if ($app->request_method() eq 'POST') {
		require MT::Entry;
		my $e = MT::Entry->load($app->{'query'}->param('entry_id'));
		my $settings = $tinyturing->blog_settings($e->blog_id);
		if ($settings->{'enabled'} && ($settings->{'rejected'} eq 'reject')
				&& !tt_test($app)) {
			return $app->handle_error('Please enter the correct letter for anti-spam verification');
		}
	}
	return &{$mt_post}(@_);
}

sub tt_test {
	my ($app) = @_;
	my $q = $app->{'query'};
	my $tt_letter = lc($q->param('tinyturing'));
	my $e = MT::Entry->load($app->{'query'}->param('entry_id'));
	my $settings = $tinyturing->blog_settings($e->blog_id);
	return 1 unless ($settings->{'enabled'});
	if ($tt_letter) {
		my $hidden = substr(crypt($tt_letter, $settings->{'salt'}), 2);
		if ($q->param($hidden)) {
			return 1;
		}
	}
	return 0;
}

sub enabled {
	my ($blog_id) = @_;
	my $settings = $tinyturing->blog_settings($blog_id);
	return $settings->{'enabled'} ? 1 : 0;
}

sub tinyturing {
	my ($ctx, $args) = @_;
	return '' unless enabled($ctx->stash('blog_id'));
	_tt_init($ctx);
	my $word = tt_word(@_);
	my $field = tt_field(@_);
	my $hidden = tt_hidden_field(@_);
	return <<HTML;
<style type="text/css">label.tinyturing { display:block; } label.tinyturing:first-letter {font-weight:bold;font-size:120%;}</style>
<label class="tinyturing">$word comment spam by typing the first letter of this sentence here:</label>
$field
$hidden
HTML
}

sub tt_start {
	my ($ctx, $args) = @_;
	_tt_init($ctx);
	return '';
}

sub tt_letter {
	my ($ctx, $args) = @_;
	defined (my $tt = $ctx->stash('TinyTuring'))
		|| return $ctx->error('MTTinyTuringStart must be called first.');
	return $tt->{'letter'};
}

sub tt_word {
	my ($ctx, $args) = @_;
	defined (my $tt = $ctx->stash('TinyTuring'))
		|| return $ctx->error('MTTinyTuringStart must be called first.');
	return ucfirst($tt->{'word'});
}

sub tt_field {
	my ($ctx, $args) = @_;
	return <<HTML;
<input type="text" name="tinyturing" style="width:30px;" />
HTML
}

sub tt_hidden_field {
	my ($ctx, $args) = @_;
	defined (my $tt = $ctx->stash('TinyTuring'))
		|| return $ctx->error('MTTinyTuringStart must be called first.');
	my $crypted = substr($tt->{'hidden'}, 2);
	return <<HTML;
<input type="hidden" name="$crypted" value="1" />
HTML
}

sub _tt_init {
	my ($ctx) = @_;
	my $tt = {};
	my $let = int(rand(26));
	$tt->{'letter'} = ${['a' .. 'z']}[$let];
	$tt->{'word'} = $words[$let];
	my $settings = $tinyturing->blog_settings($ctx->stash('blog_id'));
	$tt->{'hidden'} = crypt($tt->{'letter'}, $settings->{'salt'});
	$ctx->stash('TinyTuring', $tt);
}

sub tt_junkfilter {
	my ($comment) = @_;
	if ((ref($comment) ne 'MT::Comment') || $comment->{'TinyTuring_ok'}) {
		return ABSTAIN;
	} else {
		return (-10, 'Commenter did not enter validation letter');
	}
}

sub apply_default_settings {
	my ($plugin, $data, $scope_id) = @_;
	if ($scope_id eq 'system') {
		return $plugin->SUPER::apply_default_settings($data, $scope_id);
	} else {
		my $sys;
		for my $setting (@{$plugin->{'settings'}}) {
			my $key = $setting->[0];
			next if exists($data->{$key});
				# don't load system settings unless we need to
			$sys ||= $plugin->get_config_obj('system')->data;
			$data->{$key} = $sys->{$key};
		}
	}
}

sub blog_settings {
	my $plugin = shift;
	my ($blog_id) = @_;
	$blog_settings->{$blog_id} ||=
		$plugin->get_config_hash('blog:' . $blog_id);
	return $blog_settings->{$blog_id};
}

1;
